
from .database.meteo import MeteoDatabase
import pandas as pd
from typing import Dict
from datetime import datetime
from grassat.soil import *
from grassat.model.field_data import *



def getMeteoData(field_data: FieldData) -> pd.DataFrame:
    meteo_db = MeteoDatabase()
    meteo = meteo_db.get_meteo(field_data)
    meteo_df = pd.DataFrame(meteo)
    return meteo_df

def getIrrigations(irrigations) -> pd.DataFrame:
    # Convert irrigations to match format of irrigatie.csv
    irrigations_data = []
    if irrigations:
        for irrigation in irrigations:
            # Convert date to "days from start of year"
            date_obj = datetime.strptime(irrigation["date"], "%Y-%m-%dT%H:%M:%S.%fZ")
            start_of_year = datetime(date_obj.year, 1, 1)
            days = (date_obj - start_of_year).days
        
            # Check unit and convert dose to mm if necessary
            if irrigation["unit"] == "mm":
                dose_mm = irrigation["dose"]
            elif irrigation["unit"] == "l/m2":
                dose_mm = irrigation["dose"]  # 1 l/m2 is equivalent to 1 mm
            else:
                print(f"Skipping irrigation entry with invalid unit: '{irrigation['unit']}'.")
                continue  # Skip entry

            irrigations_data.append({"Dates": days, "mm": dose_mm})

    if len(irrigations_data) < 1:
        irrigations_data = [{"Dates": 1, "mm": 1}]  # default irrigation is set to avoid empty dataframe issue in calculations
        # TODO empty dataframe issue should be fixed

    # Create and sort the irrigations DataFrame
    irrigations_df = pd.DataFrame(irrigations_data, columns=["Dates", "mm"]).sort_values(by="Dates")

    return irrigations_df

def getFertilisations(fertilisations) -> pd.DataFrame:
    # Define conversion factors to kg/ha
    conversion_factors = {
        "KG_HA": 1,
        "M_3_HA": 1000,  # 1 cubic meter = 1000 kg
        "TON_HA": 1000,  # 1 ton = 1000 kg
        "L_HA": 1,       # 1 l (liter) = 1 kg
    }

    # Convert fertilisations to match the format of mest.csv
    fertilisations_data = []
    if fertilisations:
        for fertilisation in fertilisations:
            # Convert date to "days from start of year"
            date_obj = datetime.strptime(fertilisation["date"], "%Y-%m-%dT%H:%M:%S.%fZ")
            start_of_year = datetime(date_obj.year, 1, 1)
            days = (date_obj - start_of_year).days
            
            dose = fertilisation["products"][0]["dose"]
            dose_unit = fertilisation["products"][0]["doseUnit"]
        
            # Convert dose to kg/ha based on unit
            if dose_unit in conversion_factors:
                dose_kg_ha = dose * conversion_factors[dose_unit]
            else:
                print(f"Skipping fertilisation entry with invalid or unconvertible unit: '{dose_unit}'.")
                continue  # Skip entry

            fertilisations_data.append({"Dates": days, "kgN_ha": dose_kg_ha})

    if len(fertilisations_data) < 1:
        fertilisations_data = [{"Dates": 1, "kgN_ha": 0}] # default fertilisation is set to avoid empty dataframe issue in calculations

    # Create and sort the fertilisations DataFrame, or create empty DataFrame if no data
    fertilisations_df = pd.DataFrame(fertilisations_data, columns=["Dates", "kgN_ha"]).sort_values(by="Dates")

    return fertilisations_df

def getSoilData(geometry:dict) -> pd.DataFrame:
    soil_data = get_field_soil_data(geometry=geometry)
    return soil_data

